import orjson
import os

from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.runner_filter import RunnerFilter
from checkov.common.util.banner import banner as checkov_banner
from checkov.main import DEFAULT_RUNNERS

from cli.apiexternal import get_integration_check, push_integration_check


def file_inspection(policy_name, path):
    policy = get_integration_check(policy_name)
    if "error" in policy:
        return

    checks = policy["checks"] if "all" not in policy["checks"] else None
    skip_checks = policy["skip_checks"] if policy["skip_checks"] != [""] else None

    runner_filter = RunnerFilter(
        framework="all", checks=checks, skip_checks=skip_checks
    )
    runner_registry = RunnerRegistry(
        checkov_banner,
        runner_filter,
        *DEFAULT_RUNNERS,
    )
    if os.path.isdir(path):
        scan_reports = runner_registry.run(root_folder=path)
    elif os.path.isfile(path):
        scan_reports = runner_registry.run(files=[path])
    else:
        return

    report_jsons = []
    for report in scan_reports:
        report_jsons.append(report.get_dict())
    
    def set_default(obj):
        if isinstance(obj, set):
            return list(obj)
        raise TypeError


    if len(report_jsons) == 1:
        push_integration_check(
            {
                "scan": orjson.dumps(report_jsons[0], default=set_default).decode(),
                "policy_id": policy["id"],
                "result": len(report_jsons[0]["results"]["failed_checks"]) <= 0,
            }
        )
        return len(report_jsons[0]["results"]["failed_checks"]) <= 0
    else:
        push_integration_check(
            {
                "scan": orjson.dumps(report_jsons, default=set_default).decode(),
                "policy_id": policy["id"],
                "result": all(
                    [
                        len(report["results"]["failed_checks"]) <= 0
                        for report in report_jsons
                    ]
                ),
            }
        )
        return all(
            [len(report["results"]["failed_checks"]) <= 0 for report in report_jsons]
        )



import enum
import json
import logging
from urllib.parse import urlparse, parse_qs
from cli.util import Formatter, pretty_date, edit_entry

from cli.apiexternal import (
    get_integrations,
    add_registry,
    update_registry,
    delete_integration,
    add_cloud_integration,
    check_aws_credentials,
    create_cloudformation_template,
    create_cloudformation_stack,
    describe_cloudformation_stack,
)

log = logging.getLogger()


@enum.unique
class IntegrationTypes(enum.IntEnum):
    ANY = 0
    DOCKER_REGISTRY = 4


class DockerRegistryIntegration:
    TYPE = IntegrationTypes.DOCKER_REGISTRY

    @staticmethod
    def add(
        name,
        address,
        repository,
        username=None,
        password=None,
        type_="docker_v2",
        lookup_tag="latest",
    ):
        try:
            type_ = {"docker_hub": 1, "ecr": 2, "ecr": 3, "acr": 4, "docker_v2": 5,}[
                type_
            ]
        except KeyError:
            raise Exception(
                f"Registry type {type_} not part of (docker_hub|aws|gcr|azure|docker_v2)"
            )

        return add_registry(
            name,
            address,
            repository,
            username=username,
            password=password,
            type_=type_,
            lookup_tag=lookup_tag,
        )

    @staticmethod
    def list_():
        registries = get_integrations(type_=DockerRegistryIntegration.TYPE)
        headers = [
            "Id",
            "Name",
            "Registry Address",
            "Repository",
            "Lookup Tag",
            "Created",
            "Last Updated",
            "Status",
        ]
        rows = [
            [
                registry["id"],
                registry["name"],
                registry["address"],
                registry["repository"],
                registry["lookup_tag"],
                pretty_date(registry["created_time"]),
                pretty_date(registry["modified_time"]),
                registry["status"],
            ]
            for registry in registries
        ]
        return Formatter.table(headers, rows)

    @staticmethod
    def get(id_):
        integration = get_integrations(type_=DockerRegistryIntegration.TYPE, id_=id_)[0]
        return Formatter.integration(integration)

    @staticmethod
    def edit(id_):
        registry = get_integrations(type_=DockerRegistryIntegration.TYPE, id_=id_)[0]
        type_ = registry["type_id"]
        for key in (
            "created_by_user_id",
            "created_time",
            "added_to_anchore",
            "status",
            "failreason",
            "modified_by_user_id",
            "modified_time",
            "type_id",
            "registry_type",
        ):
            del registry[key]

        with edit_entry(json.dumps(registry, indent=4), suffix=".json") as f:
            edited = json.load(f)

        if edited == registry:
            return "No changes detected!"

        updated = update_registry(id_, edited, type_=type_)
        if updated:
            return "Updated!"
        else:
            return "Failed"

    @staticmethod
    def delete(id_):
        return delete_integration(id_, type_=DockerRegistryIntegration.TYPE)


class CloudIntegration:
    @staticmethod
    def _add_aws_integration(
        name,
        types,
        flow_log_resource_type=None,
        flow_log_resource_id=None,
        aws_role_arn=None,
    ):
        if not aws_role_arn:
            input_ = input(
                "VantagePoint will create new IAM role for this integration, do you want to proceed [Y/N]: "
            )
            if not input_.lower() == "y":
                return "Aborting!"

            log.info("Creating cloudformation template")
            cloudformation_url = create_cloudformation_template(
                integration_types=types,
                flow_log_resource_type=flow_log_resource_type,
                flow_log_resource_id=flow_log_resource_id,
            )
            template_url = cloudformation_url.split("templateURL=")[-1]
            path = template_url.split("&")[0]
            qs = parse_qs(template_url)
            log.info("Creating stack")
            # create_cloudformation_stack(path, qs["stackName"][0], qs["param_ExternalId"][0])
            stack = describe_cloudformation_stack(qs["stackName"][0])
            for output in stack["Outputs"]:
                if output["OutputKey"] == "RoleARN":
                    aws_role_arn = output["OutputValue"]
                    break
            else:  # no-break
                raise Exception("Error creating aws role")

        log.info("Checking credentials")
        if not check_aws_credentials(aws_role_arn):
            raise Exception(f'Check credentials failed on resource "{aws_role_arn}"')

        log.info("Adding integration")
        return add_cloud_integration(
            name,
            types,
            provider="aws",
            aws_role_arn=aws_role_arn,
            flow_log_resource_type=flow_log_resource_type,
            flow_log_resource_id=flow_log_resource_id,
        )

    @staticmethod
    def _add_azure_integraiton(
        name, types, azure_principal, azure_password, azure_tenant
    ):
        return add_cloud_integration(
            name,
            types,
            provider="azure",
            azure_principal=azure_principal,
            azure_password=azure_password,
            azure_tenant=azure_tenant,
        )

    @staticmethod
    def add(
        name,
        provider,
        types=[],
        aws_role_arn=None,
        flow_log_resource_type=None,
        flow_log_resource_id=None,
        azure_principal=None,
        azure_password=None,
        azure_tenant=None,
    ):
        try:
            types = [
                {"security": 1, "cloud_log": 2, "network": 5, "flow_log": 3,}[type_]
                for type_ in types
            ]
        except KeyError:
            raise Exception("Unknown integration type")

        if provider == "aws":
            # resource type and id required for flow log
            if 3 in types and not (flow_log_resource_type and flow_log_resource_id):
                raise Exception("Resource type and id is required for integration type flow_log. See --help")

            allowed_resource_types = ["VPC", "Subnet", "NetworkInterface"]
            if flow_log_resource_type and flow_log_resource_type not in allowed_resource_types:
                raise Exception(f"Resource type should be one of {str(allowed_resource_types)}")

            return CloudIntegration._add_aws_integration(name, types, flow_log_resource_type, flow_log_resource_id, aws_role_arn)

        if provider == "azure":
            return CloudIntegration._add_azure_integraiton(
                name, types, azure_principal, azure_password, azure_tenant
            )

    @staticmethod
    def list_():
        integrations = get_integrations(type_=IntegrationTypes.ANY)
        headers = ["Id", "Name", "Cloud Provider", "Created", "Last Updated", "Status"]
        rows = [
            [
                integration["id"],
                integration["name"],
                integration["cloud_provider"].title(),
                pretty_date(integration["created_time"]),
                pretty_date(integration["modified_time"]),
                "DISABLED" if integration["is_deleted"] else "ENABLED",
            ]
            for integration in integrations
            if not integration["is_deleted"]
        ]
        return Formatter.table(headers, rows)

    @staticmethod
    def get(id_):
        integration = get_integrations(type_=IntegrationTypes.ANY, id_=id_)
        return Formatter.integration(integration)

    @staticmethod
    def delete(id_):
        return delete_integration(id_, type_=IntegrationTypes.ANY)



import sys
import logging
import time
import json
from termcolor import colored
from cli.util import Formatter, spinner, pretty_date
from cli.integration import IntegrationTypes

from cli.apiexternal import (
    get_integrations,
    add_registry,
    get_images,
    add_image,
    delete_image,
    get_image_vulnerabilities,
    evaluate_image,
)

log = logging.getLogger()


class Image:
    @staticmethod
    def add(
        input_image, dockerfile=None, force=False, annotations="", autosubscribe=False
    ):
        registry = input_image.split("/")[0]
        integrations = get_integrations(type_=IntegrationTypes.DOCKER_REGISTRY)

        for integration in integrations:
            if registry != integration["address"]:
                continue

            if integration["status"] == "ACTIVE":
                break
            else:
                log.warn(
                    f"Integration \"{integration['name']}\" is still in {integration['status']} state. Images that require authorisation may fail."
                )
                break

        else:  # no-break
            log.warn(
                f'No integration for registry "{registry}" found. Images that require authorisation may fail.'
            )

        added = add_image(input_image, dockerfile, force, annotations, autosubscribe)

        return added

    @staticmethod
    def list_():
        images = get_images()
        headers = ["Image Tag(s)", "Image Digest", "Status", "Added", "Last Updated"]
        rows = [
            [
                f"{img['registry']}/{img['repo']}:{img['tag']}",
                img["digest"],
                img["analysis_status"],
                pretty_date(img["created_at"]),
                pretty_date(img["last_updated"]),
            ]
            for img in images
        ]
        return Formatter.table(headers, rows)

    @staticmethod
    def analyze(
        input_image, timeout, dockerfile=None, output=None, exit_code=None, force=False
    ):
        log.info(f'Analysing image "{input_image}" ...')
        tag = input_image.split(":")[-1]

        image = get_images(input_image)
        if not image or force:
            log.info(f'Adding image "{input_image}"')
            Image.add(input_image, dockerfile, force)
            image = get_images(input_image)
        else:
            if dockerfile:
                log.warn("Skipping docker file, image already exists")

        while timeout > 0:
            image = get_images(input_image)
            status = image["analysis_status"]
            if status == "analysing":
                continue
            elif status == "analysis_failed":
                raise Exception("Analysis failed")
            elif status == "analyzed":
                break
            time.sleep(3)
            timeout -= 3
        else:  # no-break
            raise Exception("Operation timed out")

        evaluation, status = evaluate_image(image["imageDigest"], tag)
        log.info("Finished analysing image")
        print(Formatter.image_evaluation(evaluation))

        if output:
            with open(output, "w") as f:
                f.write(json.dumps(evaluation, indent=2))

        if status == "pass":
            print(colored("PASSED!", "green"))
            if not exit_code:
                exit_code = 0
        else:
            print(colored("FAILED", "red"))
            if not exit_code:
                exit_code = 1

        sys.exit(exit_code)

    @staticmethod
    def delete(input_image, force=True):
        return delete_image(input_image, force)



import os
import logging
import sys
import requests
import subprocess
import base64
import json
import time
import getpass
import tempfile
from functools import partial
from termcolor import colored
from paramiko import client, AutoAddPolicy
from cli.apiexternal import get_vp_install_key
from cli.util import (
    ssh_connect,
    _exec_command_remote,
    _win_execute_on_computer,
    _win_get_computer_list_ad,
)


log = logging.getLogger()


class Agent:
    AGENT_NAME = "vp-agent"
    AGENT_HOME = "/opt/vantagepoint"

    def _deploy_linux(self, hosts=[], username=None, password=None):
        log.info("Setting up environment...")
        env = os.environ.copy()
        # get install key
        key = get_vp_install_key()
        env["VP_INSTALL_KEY"] = key
        # get install script
        install_script = requests.get(
            "https://artifactory.build.vantagepoint.co/artifactory/tools/client-agent-install.sh"
        ).text

        if hosts:
            ssh_client = client.SSHClient()
            ssh_client.load_system_host_keys()
            ssh_client.set_missing_host_key_policy(AutoAddPolicy())
            for host in hosts:
                log.info(
                    colored(f'Installing agent on "{host}"', "white", attrs=["bold"])
                )
                _host, port = host.split(":") if ":" in host else (host, 22)
                user, host_name = _host.split("@")
                ssh_connect(ssh_client, user, host_name, port)
                sftp_client = ssh_client.open_sftp()
                _install_script = sftp_client.open("/tmp/vp_agent_install.sh", "w")
                _install_script.write(install_script)
                _install_script.close()
                _exec_command_remote(
                    ssh_client,
                    "chmod +x /tmp/vp_agent_install.sh && /tmp/vp_agent_install.sh",
                    host,
                    env=env,
                )
                self._update_agent_config(
                    vp_install_key=key, host=host, ssh_client=ssh_client
                )
                ssh_client.close()

        else:
            if sys.platform != "linux":
                raise Exception("OS not supported")

            log.info("Installing agent...")
            with open("/tmp/vp_agent_install.sh", "w") as f:
                f.write(install_script)

            os.chmod("/tmp/vp_agent_install.sh", 755)
            process = subprocess.Popen(
                ["/bin/bash", "/tmp/vp_agent_install.sh"],
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
                env=env,
            )

            process.wait()

            # update config
            self._update_agent_config(key)

        return "Success!"

    def _update_agent_config(self, vp_install_key, host=None, ssh_client=None):
        settings_file = f"{Agent.AGENT_HOME}/conf/settings.json"
        customer_id = base64.b64decode(vp_install_key).decode("utf-8").split("::::")[0]
        encryption_key = (
            base64.b64decode(vp_install_key).decode("utf-8").split("::::")[1]
        )
        portal_host = base64.b64decode(vp_install_key).decode("utf-8").split("::::")[2]
        workspace_id = base64.b64decode(vp_install_key).decode("utf-8").split("::::")[3]

        log.info("Updating agent config ...")

        if host:
            sftp_client = ssh_client.open_sftp()
            _config = sftp_client.open(settings_file, "r")
            config = json.loads(_config.read())
        else:
            with open(settings_file, "r") as f:
                config = json.loads(f.read())

        for k, v in {
            "customer_id": customer_id,
            "customer_encryption_key": encryption_key,
            "portal_host": portal_host,
            "workspace_id": workspace_id,
        }.items():
            config[k] = v

        if host:
            _config = sftp_client.open("/tmp/vp_agent_settings.json", "w")
            _config.write(json.dumps(config))
            _config.close()
            _exec_command_remote(
                ssh_client,
                '/bin/bash -c "%s"'
                % (
                    """
                sudo systemctl stop vp-agent
                sleep 5s
                sudo mv /tmp/vp_agent_settings.json %s
                set -x
                sudo systemctl start vp-agent
                        """
                    % (settings_file)
                ),
                host,
            )
        else:
            with open("/tmp/vp_agent_settings.json", "w") as f:
                f.write(json.dumps(config))

            process = subprocess.Popen(
                [
                    "bash",
                    "-c",
                    """
                sudo systemctl stop vp-agent
                sleep 5s
                sudo mv /tmp/vp_agent_settings.json %s
                set -x
                sudo systemctl start vp-agent
                    """
                    % (settings_file),
                    Agent.AGENT_NAME,
                ],
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
            )

            process.wait()

            for line in iter(process.stdout.readline, b""):
                sys.stdout.write(line.decode("utf-8"))

    def _deploy_windows(
        self, hosts=[], use_ad=False, ad_server=None, ad_ou_dn=None, use_ssl=False
    ):
        log.info("Setting up environment...")
        env = os.environ.copy()
        # get install key
        key = get_vp_install_key()
        env["VP_INSTALL_KEY"] = key
        # get install script
        install_script_url = "https://artifactory.build.vantagepoint.co/artifactory/tools/client-agent-install.ps1"
        install_script = requests.get(install_script_url).text

        if use_ad or hosts:
            password = getpass.getpass()

            if use_ad:
                ad_user = input("AD user: ")

        if use_ad:
            log.info("Getting computer list from domain")
            l = _win_get_computer_list_ad(
                base_dn=ad_ou_dn,
                ad_domain=ad_server,
                username=ad_user,
                password=password,
            )
            hosts.append(f"{username}@{computer}" for computer in l)
        if hosts:
            for h in hosts:
                username, computer = h.split("@")
                computer = computer.split(":")
                computer_name, port = (
                    computer[0],
                    computer[1] if len(computer) > 1 else 5985,
                )
                _win_execute_on_computer(
                    computer_name,
                    username,
                    password,
                    install_script_url,
                    port=port,
                    vp_install_key=key,
                )
        else:
            if not sys.platform.startswith("win"):
                raise Exception("OS not supported")
            log.info("Installing agent...")
            with tempfile.TemporaryDirectory() as td:
                fname = os.path.join(td, "vp_agent_install.ps1")
                with open(fname, "w") as f:
                    f.write(install_script)

                process = subprocess.Popen(
                    ["powershell.exe", fname, "-VPInstallKey", key],
                    stdout=subprocess.PIPE,
                    stdin=subprocess.PIPE,
                    env=env,
                )
                process.wait()

            for line in iter(process.stdout.readline, b""):
                sys.stdout.write(line.decode("utf-8"))

    @staticmethod
    def deploy(os, hosts, use_ad, ad_server, ad_ou_dn, use_ssl):
        try:
            deploy = {
                "linux": partial(Agent()._deploy_linux, hosts),
                "windows": partial(
                    Agent()._deploy_windows, hosts, use_ad, ad_server, ad_ou_dn, use_ssl
                ),
            }[os]
        except KeyError:
            raise Exception('Unknown os "{os}"')

        return deploy()



from inspect import getdoc
from docopt import docopt
from docopt import DocoptExit


def get_docopt_help(docstring, *args, **kwargs):
    try:
        return docopt(docstring, *args, **kwargs)
    except DocoptExit:
        raise SystemExit(docstring)


class DocoptDispatcher:
    def __init__(self, command_cls, **options):
        self.command_cls = command_cls
        self.options = options

    def parse(self, argv):
        command_help = getdoc(self.command_cls) + "\n"
        options = get_docopt_help(command_help, argv, **self.options)
        command = options["COMMAND"]

        if command is None:
            raise SystemExit(command_help)

        if not hasattr(self.command_cls, command):
            print(f'Command "{command}" not found \n')
            raise SystemExit(command_help)

        handler = getattr(self.command_cls, command)
        docstring = getdoc(handler)

        command_options = get_docopt_help(
            docstring, options["ARGS"], options_first=False
        )

        return handler, command_options



import os
import requests
import logging
import base64
import boto3
from cli.util import Formatter, get_vp_config
from cli.consts import VP_CLI_VERSION
import stat
import sys
import re
import configparser
from pathlib import Path
from io import StringIO

VP_PORTAL = None
VP_TOKEN = None
VP_SECRET_KEY = None
VP_WORKSPACE = None
CREDENTIALS_FILE = f"{Path.home()}/.vantagepoint/config"

##########################################################################################################3


def load_credentials(report_err=True):
    global VP_PORTAL, VP_TOKEN, VP_SECRET_KEY, VP_WORKSPACE
    config = configparser.ConfigParser()

    VP_PORTAL = os.environ.get("VP_PORTAL", None)
    VP_TOKEN = os.environ.get("VP_TOKEN", None)
    VP_SECRET_KEY = os.environ.get("VP_SECRET_KEY", None)
    VP_WORKSPACE = os.environ.get("VP_WORKSPACE", None)

    if not all([VP_PORTAL, VP_TOKEN, VP_SECRET_KEY]):
        try:
            config.read(CREDENTIALS_FILE)
            VP_TOKEN = config.get("default", "token")
            VP_SECRET_KEY = config.get("default", "secret_key")
            VP_PORTAL = config.get("default", "portal")
            VP_WORKSPACE = config.get("default", "workspace", fallback=None)
            if not VP_PORTAL.endswith("/"):
                VP_PORTAL = "%s/" % (VP_PORTAL)
        except:
            pass

    if not all([VP_PORTAL, VP_TOKEN, VP_SECRET_KEY]) and report_err:
        raise Exception(
            "You must set configuration via 'vp_cli configure' or provide it as environment variables VP_PORTAL, VP_TOKEN, VP_SECRET_KEY."
        )


def configure():

    global VP_PORTAL, VP_TOKEN, VP_SECRET_KEY
    load_credentials(report_err=False)
    credentials = {"token": VP_TOKEN, "secret_key": VP_SECRET_KEY, "portal": VP_PORTAL}

    try:
        Path(os.path.dirname(CREDENTIALS_FILE)).mkdir(parents=True, exist_ok=True)
    except:
        pass

    VP_TOKEN = input(
        "Token [%s]: "
        % (
            "%s%s" % ("*" * 10, credentials["token"][-4:])
            if credentials["token"]
            else "*" * 14
        )
    )
    while not re.search("^[A-Za-z0-9\.\-\_]{60,140}$", VP_TOKEN):
        print("Invalid format. Try again.")
        VP_TOKEN = input(
            "Token [%s]: "
            % (
                "%s%s" % ("*" * 10, credentials["token"][-4:])
                if credentials["token"]
                else "*" * 14
            )
        )

    VP_SECRET_KEY = input(
        "Secret Key [%s]: "
        % (
            "%s%s" % ("*" * 10, credentials["secret_key"][-4:])
            if credentials["secret_key"]
            else "*" * 14
        )
    )
    while not re.search("^[a-f0-9]{48}$", VP_SECRET_KEY):
        print("Invalid format. Try again.")
        VP_SECRET_KEY = input(
            "Secret Key [%s]: "
            % (
                "%s%s" % ("*" * 10, credentials["secret_key"][-4:])
                if credentials["secret_key"]
                else "*" * 14
            )
        )

    VP_PORTAL = input(
        "Portal URL [%s]: "
        % ("%s" % (credentials["portal"]) if credentials["portal"] else "*" * 14)
    )
    while not VP_PORTAL.lower().startswith("https://"):
        print("Invalid format, must start with https://. Try again.")
        VP_PORTAL = input(
            "Portal URL [%s]: "
            % ("%s" % (credentials["portal"]) if credentials["portal"] else "*" * 14)
        )

    config = configparser.ConfigParser()
    config.readfp(StringIO("[default]"))
    config.set("default", "token", VP_TOKEN)
    config.set("default", "secret_key", VP_SECRET_KEY)
    config.set("default", "portal", VP_PORTAL)
    with open(CREDENTIALS_FILE, "w+") as configfile:
        config.write(configfile)

    os.chmod(CREDENTIALS_FILE, stat.S_IRUSR | stat.S_IWUSR)
    print("Configuration saved")


##########################################################################################################3

log = logging.getLogger()

if len(sys.argv) > 1 and not sys.argv[1].lower() == "configure":
    try:
        load_credentials()
        session = requests.Session()
        session.headers.update(
            {
                "User-Agent": f"vp-cli/v{VP_CLI_VERSION}",
                "X-VP-TOKEN": VP_TOKEN,
                "X-VP-SECRET": VP_SECRET_KEY,
                "X-VP-WORKSPACE": VP_WORKSPACE,
            }
        )
    except Exception as e:
        log.error(str(e))
        sys.exit(1)



##########################################################################################################3


def catch_exception(response):
    try:
        response.raise_for_status()
    except Exception as e:
        try:
            error_message = e.response.json()["message"]
            raise Exception(error_message)
        except:
            raise Exception("Failed!")


##########################################################################################################3


def get_integrations(type_, id_=None):
    endpoint = f"{VP_PORTAL}/api/ops_list_integrations"
    params = {"type": type_.value, "id": id_}
    response = session.get(endpoint, params=params)
    catch_exception(response)
    return response.json()["data"]


def add_registry(
    name,
    address,
    repository,
    username=None,
    password=None,
    type_=5,
    lookup_tag="latest",
):
    endpoint = f"{VP_PORTAL}/asynch_integration_items"
    integration_type = 4

    payload = {
        "name": name,
        "type_id": integration_type,
        "docker_registry_address": address,
        "docker_registry_uname": username if username else "",
        "docker_registry_password": password if password else "",
        "docker_repository_name": repository,
        "docker_registry_type": type_,
        "docker_repository_lookup_tag": lookup_tag,
    }
    response = session.post(endpoint, json=payload)
    catch_exception(response)
    return "Added!"


def update_registry(id_, registry, type_):
    endpoint = f"{VP_PORTAL}/asynch_integration_items"
    integration_type = 4

    payload = {
        "id": id_,
        "name": registry["name"],
        "type_id": type_,
        "docker_registry_address": registry["address"],
        "docker_registry_uname": registry["username"] if registry["username"] else "",
        "docker_registry_password": registry["password"]
        if registry["password"]
        else "",
        "docker_registry_type": type_,
        "docker_repository_name": registry["repository"],
        "docker_repository_lookup_tag": registry["lookup_tag"],
    }
    response = session.post(endpoint, json=payload)
    catch_exception(response)
    return "Updated!"


def delete_integration(id_, type_):
    endpoint = f"{VP_PORTAL}/integration/delete"
    response = session.delete(endpoint, params={"id": id_, "type": type_.value})
    catch_exception(response)
    return "Success"


def add_cloud_integration(
    name,
    types,
    provider,
    aws_role_arn=None,
    flow_log_resource_type=None,
    flow_log_resource_id=None,
    azure_principal=None,
    azure_password=None,
    azure_tenant=None,
):
    endpoint = f"{VP_PORTAL}/asynch_cloud_integration_items"
    payload = {
        "name": name,
        "integration_types": types,
        "cloud_provider": provider,
        "aws_role_arn": aws_role_arn,
        "flow_log_resource_type": flow_log_resource_type,
        "flow_log_resource_id": flow_log_resource_id,
        "azure-principal-name": azure_principal,
        "azure-principal-password": azure_password,
        "azure-principal-tenant": azure_tenant,
    }
    response = session.post(endpoint, json=payload)
    catch_exception(response)
    return "Added!"


def check_aws_credentials(aws_role_arn, integration_types=[]):
    endpoint = f"{VP_PORTAL}/asynch_cloud_credentials"
    payload = {"integration_types": integration_types, "aws_role_arn": aws_role_arn}
    response = session.post(endpoint, json=payload)
    catch_exception(response)
    return True if response.json()["result"] == "success" else False


def create_cloudformation_template(
    integration_types, flow_log_resource_type, flow_log_resource_id
):
    endpoint = f"{VP_PORTAL}/cloudformation/templates/integrations"
    payload = {
        "integration_types": integration_types,
        "flow_log_resource_type": flow_log_resource_type,
        "flow_log_resource_id": flow_log_resource_id,
    }
    response = session.post(endpoint, json=payload)
    catch_exception(response)
    return response.json()["cloud_formation_url"]


def create_cloudformation_stack(template_url, stack_name, external_id):
    client = boto3.client("cloudformation")
    response = client.create_stack(
        StackName=stack_name,
        TemplateURL=template_url,
        Parameters=[{"ParameterKey": "ExternalId", "ParameterValue": external_id}],
        Capabilities=["CAPABILITY_IAM"],
    )
    return response


def describe_cloudformation_stack(stack_name):
    client = boto3.client("cloudformation")
    waiter = client.get_waiter("stack_create_complete")
    waiter.wait(StackName=stack_name, WaiterConfig={"Delay": 30, "MaxAttempts": 120})
    return client.describe_stacks(StackName=stack_name)["Stacks"][0]


##########################################################################################################3


def get_images(input_image=None):
    endpoint = f"{VP_PORTAL}/api/ops_list_images"
    response = session.get(endpoint)
    catch_exception(response)
    images = response.json()["data"]
    if input_image:
        registry = input_image.split("/")[0]
        repo, tag = input_image.split("/")[-1].split(":")

        filtered = list(
            filter(
                lambda img: img["registry"] == registry
                and img["repo"] == repo
                and img["tag"] == tag,
                images,
            )
        )

        return filtered[0] if filtered else None

    else:
        return images


def add_image(
    input_image, dockerfile=None, force=False, annotations="", autosubscribe=False
):
    endpoint = f"{VP_PORTAL}/api/image_scan"
    if dockerfile:
        with open(dockerfile, "r") as f:
            dockerfile = base64.encodestring(f.read().encode())
    payload = {
        "input_image": input_image,
        "dockerfile": dockerfile.decode("utf-8") if dockerfile else "",
        "annotations": annotations,
    }
    response = session.post(
        endpoint,
        json=payload,
        params={
            "force": str(force).lower(),
            "autosubscribe": str(autosubscribe).lower(),
        },
    )
    catch_exception(response)
    return "Success"


def delete_image(input_image, force=True):
    endpoint = f"{VP_PORTAL}/api/image_scan"
    payload = {"input_image": input_image}
    response = session.delete(endpoint, json=payload, params={"force": force})
    catch_exception(response)
    return "Success"


##########################################################################################################3


def get_image_vulnerabilities(digest):
    endpoint = f"{VP_PORTAL}/ops/containers/image/{digest}/vuln"
    response = session.get(endpoint)
    response.raise_for_status()
    return response.json()["data"]


##########################################################################################################3


def evaluate_image(image_digest, tag):
    endpoint = f"{VP_PORTAL}/api/image_scan/{image_digest}/evaluate"
    response = session.post(endpoint, params={"tag": tag})
    catch_exception(response)
    _data = response.json()["data"]
    return _data, next(iter(_data.values()))[0]["status"]


##########################################################################################################3


def get_reputation_feeds(id_=None):
    endpoint = f"{VP_PORTAL}/api/intel/reputation/feeds"
    response = session.get(endpoint, params={"id": id_})
    catch_exception(response)
    return response.json()["data"]


def add_reputation_feed(feed):
    endpoint = f"{VP_PORTAL}/api/intel/reputation/feeds/form/"
    response = session.post(endpoint, data=feed)
    catch_exception(response)
    return "Saved successfully" in response.text


def update_reputaiton_feed(id_, feed):
    endpoint = f"{VP_PORTAL}/api/intel/reputation/feeds/form/"
    payload = {
        "id_": id_,
        "name": feed["name"],
        "risk_score": feed["risk_score"],
        "entry_type": feed["type"],
        "entries": ",".join(feed.get("entries", [])),
        "url": feed.get("url", ""),
    }
    feed["id"] = id_
    response = session.post(endpoint, data=payload)
    catch_exception(response)
    return "Saved successfully" in response.text


def delete_reputation_feed(id_):
    endpoint = f"{VP_PORTAL}/api/intel/reputation/custom_feed/delete"
    payload = {"id": id_}
    response = session.delete(endpoint, json=payload)
    catch_exception(response)
    return "Success"


##########################################################################################################3


def get_workspaces():
    response = session.get(f"{VP_PORTAL}/api/workspaces")
    catch_exception(response)
    return response.json()["data"]


##########################################################################################################3


def file_and_event_search(endpoint, query):
    response = session.get(f"{VP_PORTAL}{endpoint}", params={"query": query})
    catch_exception(response)
    return response.json()["data"]


##########################################################################################################3


def get_vp_install_key():
    endpoint = f"{VP_PORTAL}/api/deploy/vp_install_key"
    response = session.get(endpoint)
    catch_exception(response)
    return response.json()["data"]


########################################################################################

def push_integration_check(data):
    session.post(f"{VP_PORTAL}/api/inspection/push", json=data)
    return True


def get_integration_check(policy):
    response = session.get(f"{VP_PORTAL}/api/inspection/get/{policy}")
    return response.json()
